#!/etc/opt/openit/perl

BEGIN {
	if ( $^O ne 'MSWin32' ) {
		push(@INC, "/opt/openit/lib" );
        $ENV{'OPENIT_CFG'} = "/etc/opt/openit/openit.cfg"; 
	} 
	push(@INC, $ENV{'ALTUS'} . "/src/perl/source" ); 
}

use strict;
use Data::Dumper;
use OpeniT::Config;
use File::Basename;
use Getopt::Long;
use File::Copy;
my $opt_debug;

unless ( GetOptions( "debug"          => \ $opt_debug )) {
    &print_usage;
}


my $CONFIG_DIR = &OpeniT::Config::CONFIG_DIR() . "/filtrator/";
my @filters = (
    "$CONFIG_DIR/olap-license-total-usage.conf", #102
    "$CONFIG_DIR/olap-user-concurrency.conf", #94
    "$CONFIG_DIR/olap-raw-hourly.conf", #93
    "$CONFIG_DIR/olap-group-concurrency.conf" #95
);
for my $filter (@filters) { 
    my $filter_name = basename $filter;
    &info("Reading $filter_name");
    my @lines = read_file( $filter );
    my @processlist = getFilterProcesses(\@lines );

     if ($filter_name eq "olap-license-total-usage.conf") { 
        if( grep { /CalcSum:PricingStructure/ } @lines) {
		    &info("Calcsum Pricing Structure already added.");
        } else {
            &info("Calcsum Pricing Structure will be added to $filter_name");
            &CalcSumPricingStructure(\@processlist);
            &CalcSumPricingStructureDef(\@lines);
        }
     }

    if ( grep { /MinMax/ } @lines) {
        &info("Updating MinMax.compare fields");
		&MinMaxUpdate(\@lines);
	}
    
    &beautifyFilter(\@processlist, \@lines);
    save_file( "$filter" ,@lines);
}

sub beautifyFilter() {
    my ($processlist, $lines ) = @_;

     ## Insert beautified list 
    my @processes = split /"\n"/, &beautifyProcesses(\@$processlist);
    my $process_index = locate_line( 0 , "filter-process", \@$lines );
    my $def_index = locate_line( 0 , "filter-defs", \@$lines );

    push @processes , "\t}\n";
    unshift @processes, "\t{\n";

    &debug ("process1:" .  $process_index );
    &debug ("defs1:" . $def_index  );
    splice @$lines, ($process_index+=1), ($def_index -$process_index);
    splice @$lines, $process_index, 0, @processes;

}

sub CalcSumPricingStructure() {
    my ($processlist) = @_;
    my ($index)  = grep { @$processlist[$_] =~ /MinMax/ } 0 ..@$processlist;
    splice @$processlist, ($index + 1), 0 , "CalcSum:PricingStructure"; 
}

sub CalcSumPricingStructureDef() {
    my ($lines) = @_;
    my $index = 0;
    my @ps_def = split /"\n"/, &getPricingStructureDef();
    $index = locate_line( $index, "filter-defs", \@$lines );
    $index = locate_line( $index, "{", \@$lines );
    $index +=1;
    splice @$lines, $index, 0, @ps_def;
}

sub MinMaxUpdate () {
    my ($lines) = @_;
    my $index = 0;

    $index = locate_line( $index, "filter-process", \@$lines );
    $index = locate_line( $index, "filter-defs", \@$lines );
    $index = locate_line( $index, "MinMax", \@$lines );
    $index = locate_line( $index, "compare", \@$lines );
    
    if( @$lines[$index] =~ /product.*pricing_structure/ ) {
        &debug( "'pricing_structure' is already added to MinMax.compare" );
        return;
	} 
    
    @$lines[$index] = "			compare(String)=\"product,package,feature,feature_version,pricing_structure\"{}\n";
    &debug( "'pricing_structure' will be added to MinMax.compare" );
    

}
sub beautifyProcesses() { 
    my ($processes) = @_;
    my $index = 0;
    my $beatified = "";
    for my $process (@$processes) {

        my $tabs = generateTabs($index);
        $beatified .= "$tabs$process()=\"\"";
        if ( scalar @$processes != $index +1) {
            $beatified .= "\n$tabs\{\n";
        }
        $index += 1;
    }
    $beatified .= "{}\n";
    my $size = scalar @$processes;
    $size-=2;
    for my $ind ( reverse 0 .. $size ) {
        my $tabs = generateTabs($ind);
        $beatified .= "$tabs\}\n";
    }
    return $beatified;
}


sub generateTabs() {
    my ($count) = @_;
    my $tabs = "";
    $count+=1;
    foreach  ( 0 .. $count ) { $tabs .= "\t"};
    return $tabs
}


sub getFilterProcesses() {
    my ($lines) = @_;
    my @processes = ();
  
    my $process_index = locate_line( 0 , "filter-process", \@$lines );
    my $def_index = locate_line(  0 , "filter-defs", \@$lines );
    $process_index+=1;
    $def_index-=1;

    &debug ("process:" .  $process_index );
    &debug ("defs:" . $def_index  );

    for my $index ($process_index .. $def_index ) {

        if ( @$lines[$index] =~ /\S*\(\)=/ ) {
            my $process = @$lines[$index];
            chomp $process;
            $process =~ s/^\s+|\s+$//g;
            $process =~ s/[=()""{}]+//g;
            push @processes, "$process";
        }
    }
    return @processes;
}

sub locate_line {
	my ( $start, $find, $lines_ref ) = ( @_ );
	my ( $index ) = grep { @$lines_ref[$_] =~ /$find/ } $start ..@$lines_ref;
	return $index;
}

sub read_file {
	my ( $cfg ) = @_;
    if (!-s $cfg) {
        die "ERROR: File $cfg is empty. Nothing to read";
    }
	open( IN, $cfg ) || die "ERROR: Unable to read cfg '$cfg'!\n";
	my @lines = <IN>;
	close( IN );
	return @lines;
}


sub info {
    my ($msg) = @_;
	print(" -- $msg\n");
}
sub debug {
    my ($msg) = @_;
    if (defined $opt_debug) {
        print("DEBUG: $msg\n");
    }
	
}

sub save_file {
	my ( $cfg, @lines ) = @_;
	open( OUT, ">$cfg.temp" );
	print OUT @lines;
	close( OUT );

    copy("$cfg.temp","$cfg") or die "Copy failed: $!";
    unlink "$cfg.temp";
}


sub getPricingStructureDef() {

    return <<"END_PRICING_STRUCTURE_DEF";

        CalcSum:PricingStructure()=""
        {
            active(Bool)="true"{}
            filter(String)="CalcSum"{}
            compare(String)="product,feature,feature_version,pricing_structure,origsource"
            {
                field(String)="data_type"{}
                positive(Bool)="true"{}
                pattern(String)="DSLS"{}
            }
            remove(String)="pricing_structure"{}
            summarize(String)="licenses"{}
        }
END_PRICING_STRUCTURE_DEF
}

