#!/etc/opt/openit/perl

# Applies the following updates to data_generation_config:
# Adds this entry (124:License model user concurrency:license-model-user-concurrency,P1H:on) in the licpoll2 archtype section if it is not yet existing
# Adds this entry (125:License model total use:license-model-total-usage,P1H:on) in the licpoll2 archtype section if it is not yet existing

BEGIN {
	if ( $^O ne 'MSWin32' ) {
		push(@INC, "/opt/openit/lib" );
        $ENV{'OPENIT_CFG'} = "/etc/opt/openit/openit.cfg"; 
	} 
	push(@INC, $ENV{'ALTUS'} . "/src/perl/source" ); 
}

## OpeniT Modules
use OpeniT::Config;
use strict;

my $CONFIG_DIR = &OpeniT::Config::CONFIG_DIR();
my $file = $CONFIG_DIR . "/data_generation_config";
my @lines = read_file( $file );

print " Modifying $file\n";

insert_DT124();
insert_DT125();

save_file( $file, @lines );

exit( 0 );

# Subroutines -----------
sub insert_DT124()
{
	my $datatype = "124";
	my $insert = "124:License model user concurrency:license-model-user-concurrency,P1H:on\n";
	insert_datatype( $datatype, $insert );
}

sub insert_DT125()
{
	my $datatype = "125";
	my $insert = "125:License model total use:license-model-total-usage,P1H:on\n";
	insert_datatype( $datatype, $insert );
}

sub insert_datatype
{
	my ( $defs, $block ) = (@_);
	my $index = 0;

	$index = locate_line( $index, "TYPE:licpoll2", \@lines );
	my $defs_index = locate_line( $index, $defs, \@lines );
	if( $index < $defs_index )
	{
		print " -- Datatype $defs already exist in data_generation_config\n";
	}
	else
	{
		@lines[$index] .= $block;
		print " -- Successfully inserted $defs\n";
	}
}

sub read_file {
	my ( $cfg ) = @_;
	open( IN, "$cfg" ) || die " -- ERROR: Unable to read '$cfg'!\n";
	my @lines = <IN>;
	close( IN );
	return @lines;
}

sub locate_line {
	# For carets and colons, use the . character to represent them
	# For the 3rd argument, always add a \ as prefix to the array
	my ( $start, $find, $lines_ref ) = (@_);
	my ( $index ) = grep { @$lines_ref[$_] =~ /$find/ } $start ..@$lines_ref;
	return $index;
}

sub save_file {
	my ( $cfg, @lines ) = @_;
	open( OUT, ">$cfg" );
	print OUT @lines;
	close( OUT );
}
