#!/etc/opt/openit/perl

BEGIN {
	if ( $^O ne 'MSWin32' ) {
		push(@INC, "/opt/openit/lib" );
        $ENV{'OPENIT_CFG'} = "/etc/opt/openit/openit.cfg"; 
	} 
	push(@INC, $ENV{'ALTUS'} . "/src/perl/source" ); 
}

## OpeniT Modules
use OpeniT::Config;
use strict;

my $CONFIG_DIR = &OpeniT::Config::CONFIG_DIR();
my $file = $CONFIG_DIR . "/filtrator/olap-raw-hourly.conf";
my @lines = read_file( $file );
my $index = 0;

print " Modifying $file\n";

check_corrections();
rawreport_def();
save_file( $file, @lines );


sub check_corrections
{
	if( grep { /Corrections/ } @lines )
	 {
		print " -- '" . $file . "' already has Corrections.\n";
		return;
	}
	else
	{
		corrections_proc();
		corrections_def();
	}
}

sub corrections_proc
{
	$index = 0;
	$index = locate_line( $index, "License2Action", \@lines );
	$index--;
	@lines[$index] =~ s/{\n/{\n\t\tCorrections()=""\n\t\t{\n/;

	$index = 0;
	$index = locate_line( $index, "filter-defs", \@lines );	
	$index--;	
	@lines[$index] =~ s/}\n/\t}\n\t}\n/;
}

sub rawreport_def
{
	$index = 0;
	$index = locate_line( $index, "filter-defs", \@lines );
	my $defs_index = locate_line( $index, "RawReport:olap-raw-hourly", \@lines );

	if( $index < $defs_index )
	{
	 	my $open = $defs_index+1;
	 	my $close = $open;

	 	while ( @lines[$close] !~ /([^{\w]})/ )
	 	{
	 		$close++;
	 	}	 	

	 	my $object_index = locate_line( $open, "fields", \@lines );
	 	print " -- Checking object: fields\n";
		if ( $close > $object_index )
	 	{
	 		modify_filterdefs( $object_index, "fields", "fields(String)=\"start:duration:product:package:feature:feature_version:entry_version:featureset:featureset_version:primetime:user:host:license_type:used_licenses:daemon:licenses:license_manager:'1':elapsed_time:clientport:offline_timeout:project_path:pricing_structure\"{}");
	 	}
	 	else
	 	{
	 		@lines[$close-1] .= "\t\tfields(String)=\"start:duration:product:package:feature:feature_version:entry_version:featureset:featureset_version:primetime:user:host:license_type:used_licenses:daemon:licenses:license_manager:'1':elapsed_time:clientport:offline_timeout:project_path:pricing_structure\"{}\n";
			print " -- Done inserting object.\n";
	 	}

	 	my $object_index = locate_line( $open, "file-name", \@lines );
	 	print " -- Checking object: file-name\n";
		if ( $close > $object_index )
	 	{
	 		modify_filterdefs( $object_index, "file-name", "file-name(String)=\"olap-raw\"{}");
	 	}
	 	else
	 	{
	 		@lines[$close-1] .= "\t\tfile-name(String)=\"olap-raw\"{}\n";
			print " -- Done inserting object.\n";
	 	}
	}
	else
	{
		print " -- RawReport() doesn't exist\n";
	}
}

sub corrections_def
{
	my $title = "Corrections()";
	my $insert = 	
	"\t\tCorrections()=\"\"
	\t{
	\t\tfilter(String)=\"Corrections\"{}
	\t\ttypes()=\"\"
	\t\t{\n
	\t\t\tmissing-first-poll(Bool)=\"true\"{}
	\t\t}
	\t}\n";

	insert_filterdefs( $title, $insert );
}

sub modify_filterdefs
{
	my ( $defs_index, $pattern, $insert ) = (@_);
	$defs_index = locate_line( $defs_index, $pattern, \@lines );

	my ($string) = @lines[$defs_index] =~ /([^\t\b\n]+)/;

	if ( $string eq $insert )
	{
	 	print " -- $pattern is already updated.\n";
	}
	else
	{
	 	@lines[$defs_index] =~ s/.\S*//g;
	 	@lines[$defs_index-1] .= "\t\t\t$insert";
	 	print " -- Done updating $pattern\n";
	}
}

sub insert_filterdefs
{
	my ( $defs, $block ) = (@_);

	print " -- Inserting $defs...\n";

	$index = 0;
	$index = locate_line( $index, "filter-defs", \@lines );
	my $defs_index = locate_line( $index, $defs, \@lines );
	if( $index < $defs_index )
	{		
		print " -- $defs already exist\n";
	}
	else
	{
		@lines[$index+1] .= $block;
		print " -- Done inserting $defs\n";
	}
}

sub read_file {
	my ( $cfg ) = @_;
	open( IN, "$cfg" ) || die " -- ERROR: Unable to read '$cfg'!\n";
	my @lines = <IN>;
	close( IN );
	return @lines;
}

sub locate_line {
	# For carets and colons, use the . character to represent them
	# For the 3rd argument, always add a \ as prefix to the array
	my ( $start, $find, $lines_ref ) = (@_);
	my ( $index ) = grep { @$lines_ref[$_] =~ /$find/ } $start ..@$lines_ref;
	return $index;
}

sub save_file {
	my ( $cfg, @lines ) = @_;
	open( OUT, ">$cfg" );
	print OUT @lines;
	close( OUT );
}
